import inspect
import sys
from konira.exc             import KoniraFirstFail, KoniraNoSkip
from konira.util            import StopWatch, get_class_name
from konira.collector       import globals_from_execed_file
from konira.output          import TerminalWriter, ExcFormatter, out_footer


class Runner(object):

    def __init__(self, paths, config):
        self.config         = config
        self.paths          = paths
        self.failures       = []
        self.errors         = []
        self.total_cases    = 0
        self.total_failures = 0
        self.total_errors   = 0
        self.total_skips    = 0
        self.class_name     = config.get('class_name')
        self.method_name    = config.get('method_name')
        self.write          = sys.__stdout__.write
        self.writer         = TerminalWriter(config.get('dotted'))


    def run(self):
        self.timer = StopWatch()
        for f in self.paths:
            try:
                classes = self.classes(f)
            except Exception, e:
                self.total_errors += 1
                self.errors.append(
                    dict(
                        failure   = sys.exc_info(),
                        exc_name  = e.__class__.__name__
                       ) 
                    )
                continue
            try:
                for case in classes:
                    self.run_suite(case)
            except KoniraFirstFail:
                break

        self.elapsed = self.timer.elapsed()


    def run_suite(self, case):
        # Initialize the test class
        suite = case()

        # check test environment setup
        environ = TestEnviron(suite)

        methods = self.methods(suite)
        if not methods: return

        # Name the class
        self.writer.out_case(suite.__class__.__name__)

        # Are we skipping?
        if self.safe_skip_call(environ.set_skip_if):
            self.writer.skipping()
            return

        # Set before all if any
        self.safe_environ_call(environ.set_before_all)

        for test in methods:
            self.total_cases += 1

            # Set before each if any
            self.safe_environ_call(environ.set_before_each)

            try:
                getattr(suite, test)()
                self.writer.green_spec(test)
                
            except BaseException, e:
                trace = inspect.trace()[1]
                self.total_failures += 1
                self.writer.red_spec(test)
                self.failures.append(
                    dict(
                        failure  = sys.exc_info(),
                        trace    = trace,
                        exc_name = e.__class__.__name__
                       ) 
                    )
                if self.config.get('first_fail'):
                    raise KoniraFirstFail

            # Set after each if any
            self.safe_environ_call(environ.set_after_each)

        # Set after all if any
        self.safe_environ_call(environ.set_after_all)


    def safe_environ_call(self, env_call):
        try:
            env_call()
        except Exception, e:
            self.errors.append(
                dict(
                    failure   = sys.exc_info(),
                    exc_name  = e.__class__.__name__
                   ) 
                )


    def safe_skip_call(self, env_call):
        try:
            skip = env_call()
            return True
        except KoniraNoSkip:
            return False
        except Exception:
            return False


    # XXX This is probably the wrong spot for this guy
    def report(self):
        self.write('\n')
        if self.failures:
            format_exc = ExcFormatter(self.failures, self.config)
            format_exc.output_failures()
        if self.errors:
            format_exc = ExcFormatter(self.errors, self.config)
            format_exc.output_errors()
        out_footer(self.total_cases, self.total_failures, self.elapsed)


    def classes(self, filename):
        if self.class_name:
            classes = [i for i in self._collect_classes(filename) 
                        if self.class_name == get_class_name(i)]
        else:
            classes = [i for i in self._collect_classes(filename)]

        return classes


    def methods(self, suite):
        if self.method_name:
            methods = [i for i in self._collect_methods(suite) 
                        if i == self.method_name]
        else:
            methods = self._collect_methods(suite)

        return methods


    def _collect_classes(self, path):
        global_modules = map(globals_from_execed_file, [path])
        return [i for i in global_modules[0].values() if callable(i) and i.__name__.startswith('Case_')]


    def _collect_methods(self, module):
        invalid = ['_before_each', '_before_all', '_after_each', '_after_all']
        return [i for i in dir(module) if not i.startswith('_') and i not in invalid and i.startswith('it_')] 



class TestEnviron(object):
    """
    Checks for all test setup calls and sets a boolean
    flag for each.
    This approach avoids the runner to be checking getattr
    for every time since we alredy did at the beginning.
    """


    def __init__(self, suite):
        self.suite           = suite
        self.has_skip_if     = self._skip_if
        self.has_before_all  = self._before_all
        self.has_before_each = self._before_each
        self.has_after_all   = self._after_all
        self.has_after_each  = self._after_each


    @property
    def _skip_if(self):
        if hasattr(self.suite, '_skip_if'):
            return True
        return False


    @property
    def _before_all(self):
        if hasattr(self.suite, '_before_all'):
            return True
        return False


    @property
    def _before_each(self):
        if hasattr(self.suite, '_before_each'):
            return True
        return False


    @property
    def _after_all(self):
        if hasattr(self.suite, '_after_all'):
            return True
        return False


    @property
    def _after_each(self):
        if hasattr(self.suite, '_after_each'):
            return True
        return False
    

    def set_skip_if(self):
        if self.has_skip_if:
            getattr(self.suite, '_skip_if')()
        else:
            raise KoniraNoSkip


    def set_before_all(self):
        if self.has_before_all:
            getattr(self.suite, '_before_all')()


    def set_before_each(self):
        if self.has_before_each:
            return getattr(self.suite, '_before_each')()


    def set_after_all(self):
        if self.has_after_all:
            return getattr(self.suite, '_after_all')()


    def set_after_each(self):
        if self.has_after_each:
            return getattr(self.suite, '_after_each')()
                




import time


def name_convertion(name):
    name = name.replace('_', ' ').replace('Case', '')
    name = name.capitalize().strip() 
    return name



def get_class_name(class_name):
    try:
        name = str(class_name).split("'")[1]
    except IndexError:
        return class_name
    if name.startswith('Case_'):
        return name



class StopWatch(object):
 

    def __init__(self):
        self.start = time.time()


    def elapsed(self):

        _elapsed = str(time.time() - self.start)
        return _elapsed[:5]


runner_options = dict(
        first_fail = False,
        capturing  = True,
        traceback  = False,
        dotted     = False
        )

     



from tokenize           import NAME, OP, STRING, generate_tokens
from encodings          import utf_8
import codecs
import cStringIO
import encodings
import tokenize



def quote_remover(string):
    _string = string.replace(",", "").replace(".", "")
    return _string.replace("'", "")


def valid_method_name(token):
    transform = token.strip().replace(" ", "_").replace("\"","" )
    return "it_%s" % quote_remover(transform)


def valid_class_name(token):
    transform = token.strip().replace(" ", "_").replace("\"","" )
    return "Case_%s" % quote_remover(transform)
    

def initial_imports():
    case_imports = []
    import_raise = case_imports.extend(([NAME, 'import'],[NAME, 'konira']))
    return case_imports


def translate(readline):
    result     = []
    last_kw    = None
    last_token = None
    last_type  = None
    descr_obj  = False

    # add imports
    result.extend(initial_imports())

    for tokenum, value, _, _, _ in generate_tokens(readline):

        # From Describe to class - includes inheritance
        if tokenum == NAME and value == 'describe':
            last_kw = 'describe'
            result.extend(([tokenum, 'class'],))
        elif tokenum == STRING and last_token == 'describe':
            last_kw   = 'describe'
            descr_obj = True
            result.extend(([NAME, valid_class_name(value)],))

        elif tokenum == OP and value == ',' and last_type == STRING and last_kw == 'describe':
            if descr_obj:
                result.extend(([OP, '('],))
                last_kw   = 'describe'
                descr_obj = True

        elif tokenum == NAME and last_type == OP and last_kw == 'describe':
            if descr_obj:
                result.extend(([NAME, value],
                               [OP, ')'],))
                last_kw   = None
                descr_obj = False

        elif last_type == STRING and last_kw == 'describe':
            if descr_obj:
                result.extend(([OP, '('],
                               [NAME, 'object'],
                               [OP, ')'],
                               [OP, ':'],))
                last_kw   = None
                descr_obj = False

        # Skip if Constructors
        elif tokenum == NAME and value == 'skip':
            result.extend(([tokenum, 'def'],))

        elif tokenum == NAME and last_token == 'skip' and value == 'if':
            result.extend(([tokenum, '_skip_if'],
                           [OP, '('],
                           [NAME, 'self'],
                           [OP, ')']))

        # Before Constructors
        elif tokenum == NAME and value == 'before':
            result.extend(([tokenum, 'def'],))

        elif tokenum == NAME and last_token == 'before':
            result.extend(([tokenum, '_before_%s' % value],
                           [OP, '('],
                           [NAME, 'self'],
                           [OP, ')']))

        # After Constructors
        elif tokenum == NAME and value == 'after':
            result.extend(([tokenum, 'def'],))

        elif tokenum == NAME and last_token == 'after':
            result.extend(([tokenum, '_after_%s' % value],
                           [OP, '('],
                           [NAME, 'self'],
                           [OP, ')']))

        # From it to def
        elif tokenum == NAME and value == 'it':
            result.extend(([tokenum, 'def'],))
        elif tokenum == STRING and last_token == 'it':
            result.extend(([tokenum, valid_method_name(value)],
                           [OP, '('],
                           [NAME, 'self'],
                           [OP, ')'],))

        # From raises to with konira.tools.raises
        elif tokenum == NAME and value == 'raises':
            result.extend(([tokenum, 'with konira.tools.raises'],))

        elif tokenum == NAME and last_token == 'raises':
            result.extend(([OP, '('],
                           [NAME, value],
                           [OP, ')'],))

        else:
            result.append([tokenum, value])
        last_token = value
        last_type  = tokenum
    
    return result



class StreamReader(utf_8.StreamReader):
    def __init__(self, *args, **kwargs):
        codecs.StreamReader.__init__(self, *args, **kwargs)
        data = tokenize.untokenize(translate(self.stream.readline))
        self.stream = cStringIO.StringIO(data)



def search_function(s):
    if s!='konira': return None
    utf8=encodings.search_function('utf8')
    return codecs.CodecInfo(
        name='konira',
        encode = utf8.encode,
        decode = utf8.decode,
        incrementalencoder=utf8.incrementalencoder,
        incrementaldecoder=utf8.incrementaldecoder,
        streamreader=StreamReader,
        streamwriter=utf8.streamwriter)



codecs.register(search_function)





import inspect
import difflib



class DontReadFromInput(object):
    """Temporary stub class.  Ideally when stdin is accessed, the
    capturing should be turned off, with possibly all data captured
    so far sent to the screen.  This should be configurable, though,
    because in automated test runs it is better to crash than
    hang indefinitely.
    """
    msg = "reading from stdin while output is captured (using pdb?)"


    def flush(self, *args):
        raise KoniraIOError(self.msg)


    def write(self, *args):
        raise KoniraIOError(self.msg)


    def read(self, *args):
        raise KoniraIOError(self.msg)


    readline  = read
    readlines = read
    __iter__  = read



class KoniraExecutionError(Exception):


    def __init__(self, exc_name, filename, lineno, msg, exc):
        self.exc_name = exc_name
        self.msg      = msg
        self.filename = filename
        self.lineno   = lineno
        self.exc      = exc
        Exception.__init__(self, msg)



class KoniraNoSkip(Exception):


    def __init__(self, msg=''):
        Exception.__init__(self, msg)



class KoniraReassertError(Exception):


    def __init__(self, msg=''):
        Exception.__init__(self, msg)



class KoniraFirstFail(Exception):


    def __init__(self, msg=''):
        Exception.__init__(self, msg)



class KoniraIOError(Exception):


    def __init__(self, msg):
        Exception.__init__(self, msg)



class Source(object):


    def __init__(self, trace):
        self.trace    = trace
        self.line     = self.get_assert_line
        self.operand  = self.get_operand
        self.is_valid = True
        self._locals  = self.get_locals
        if not self.line or not self.operand:
            self.is_valid = False


    @property
    def get_locals(self):
        frame = self.trace[0]
        return inspect.getargvalues(frame).locals


    @property
    def get_assert_line(self):
        line = self.trace[-2][0]
        if 'assert' in line:
            return line.replace('assert', '').strip()
        return False


    @property
    def get_operand(self):
        operators = ['==', '!=', '>', '<', 
                     '>=', '<=', ' is ', ' not in ']
        operator = [i for i in operators if i in self.line]
        if len(operator) > 1: # probably <, >, mixed with <= >= operators 
            return operator[-1]
        if operator:
            return operator[0]


    @property
    def _left_text(self):
        operator = self.operand
        return self.line.split(operator)[0].strip()


    @property
    def _right_text(self):
        operator = self.operand
        return self.line.split(operator)[1].strip()


    @property
    def right_value(self):
        right = self._locals.get(self._right_text)
        if right:
            return right
        return self._eval(self._right_text)


    @property
    def left_value(self):
        left = self._locals.get(self._left_text)
        if left:
            return left
        return self._eval(self._left_text)


    def _eval(self, code):
        return eval(code, None, self._locals)



def konira_assert(trace):
    source  = Source(trace)
    if source.is_valid:
        try:
            left     = source.left_value
            right    = source.right_value
            operand  = source.operand
            line     = source.line
        except NameError:
            return None

        # At this point we have tried everything we can to
        # get a valid comparison so return to basic Assertion
        # to avoid a huge traceback
        except Exception:
            return None

        try:
            reassert = assertrepr_compare(operand, left, right)
        except KoniraReassertError:
            return None

        if reassert:
            return reassert
        return assert_description(operand, left, right, line)



def assert_description(op, left, right, line):
    explanation = [line]
    explanation.append('%s %s %s' % (left, op, right))
    return explanation

    

def assertrepr_compare(op, left, right):
    """return specialised explanations for some operators/operands"""
    width      = 80 - 15 - len(op) - 2 # 15 chars indentation, 1 space around op
    left_repr  = left
    right_repr = right
    summary    = '%s %s %s' % (left_repr, op, right_repr)

    issequence = lambda x: isinstance(x, (list, tuple))
    istext     = lambda x: isinstance(x, basestring)
    isdict     = lambda x: isinstance(x, dict)
    isset      = lambda x: isinstance(x, set)

    explanation = None
    try:
        if op == '==':
            if istext(left) and istext(right):
                explanation = _diff_text(left, right)
            elif issequence(left) and issequence(right):
                explanation = _compare_eq_sequence(left, right)
            elif isset(left) and isset(right):
                explanation = _compare_eq_set(left, right)
            elif isdict(left) and isdict(right):
                explanation = _diff_text(str(left), str(right))
        elif op == ' not in ':
            if istext(left) and istext(right):
                explanation = _notin_text(left, right)
    except Exception, e:
        raise KoniraReassertError(e)

    if not explanation:
        return None

    # Don't include pageloads of data, should be configurable
    if len(''.join(explanation)) > 80*8:
        explanation = ['Detailed information too verbose, truncated']

    return [summary] + explanation


def _diff_text(left, right):
    """Return the explanation for the diff between text

    This will skip leading and trailing characters which are
    identical to keep the diff minimal.
    """
    explanation = []
    i = 0 # just in case left or right has zero length
    for i in range(min(len(left), len(right))):
        if left[i] != right[i]:
            break
    if i > 42:
        i -= 10                 # Provide some context
        explanation = ['Skipping %s identical '
                       'leading characters in diff' % i]
        left  = left[i:]
        right = right[i:]
    if len(left) == len(right):
        for i in range(len(left)):
            if left[-i] != right[-i]:
                break
        if i > 42:
            i -= 10     # Provide some context
            explanation += ['Skipping %s identical '
                            'trailing characters in diff' % i]
            left  = left[:-i]
            right = right[:-i]
    explanation += [line.strip('\n')
                    for line in difflib.ndiff(left.splitlines(),
                                                     right.splitlines())]
    return explanation


def _compare_eq_sequence(left, right):
    explanation = []
    for i in range(min(len(left), len(right))):
        if left[i] != right[i]:
            explanation += ['At index %s diff: %r != %r' %
                            (i, left[i], right[i])]
            break
    if len(left) > len(right):
        explanation += ['Left contains more items, '
            'first extra item: %s' % (left[len(right)],)]
    elif len(left) < len(right):
        explanation += ['Right contains more items, '
            'first extra item: %s' % (right[len(left)],)]
    return explanation # + _diff_text(py.std.pprint.pformat(left),
                       #             py.std.pprint.pformat(right))


def _compare_eq_set(left, right):
    explanation = []
    diff_left   = left - right
    diff_right  = right - left
    if diff_left:
        explanation.append('Extra items in the left set:')
        for item in diff_left:
            explanation.append(repr(item))
    if diff_right:
        explanation.append('Extra items in the right set:')
        for item in diff_right:
            explanation.append(repr(item))
    return explanation


def _notin_text(term, text):
    index        = text.find(term)
    head         = text[:index]
    tail         = text[index+len(term):]
    correct_text = head + tail
    diff         = _diff_text(correct_text, text)
    newdiff      = ['%s is contained here:' % repr(term, maxsize = 42)]
    for line in diff:
        if line.startswith('Skipping'):
            continue
        if line.startswith('- '):
            continue
        if line.startswith('+ '):
            newdiff.append('  ' + line[2:])
        else:
            newdiff.append(line)
    return newdiff




from distutils.core import setup

setup(
    name             = "konira",
    version          = "0.0.1",
    packages         = ['konira'],
    scripts          = ['bin/konira'],
    author           = "Alfredo Deza",
    author_email     = "alfredodeza [at] gmail.com",
    description      = "A DSL Testing Framework for nicer, (beautiful!) readable BDD tests",
    license          = "MIT",
    keywords         = "test, readable, testrunner, bdd",
    long_description = """

**now with 0 calorie syntactic sugar!**


Konira - A Python DSL Testing Framework
=======================================
Konira is a tool that allows you to write minimalistic
descriptions for testing scenarios and unit tests.

It is flexible enough to accommodate Behavioral Driven 
Development as well as traditional Unit Testing. 



What does a test case look like?
------------------------------------


::

    describe "a very simple test case for my_module":

        it "has a foo property that is True":
            assert my_module.foo


But this is not valid Python!
---------------------------------

I hear you. It is a DSL. All tests need to specify the ``konira`` encoding
at the top and they can be executed with the included command line tool.

fast and readable action in the terminal
--------------------------------------------


::

    $ konira
    
    a very simple test case for my_module
        It has a foo property that is True
    

    All specs passed in 0.000 secs.


Are you sold yet?
---------------------

Full docs can be found at http://konira.rtfd.org
If you have any ideas or suggestions ping me @alfredodeza

The name
------------

*Konira* is one of the names of an ancient Inca God. Konira Wirakocha diguised 
as a traveler in rags. A trickster, a prankster. No one knew who he was, and the 
people he passed called him names. Yet as he walked, he created. With a word he 
made the fields and terraced hillsides. Dropping a reed blossom, he made water flow.

This is a DSL, it looks like Python - it mostly is, but it packs simplicity and makes
testing look good and easy.
"""

)



from konira.exc import KoniraReassertError


def raises(cls=Exception, message=None):
    return AssertRaises(cls, message)



class AssertRaises(object):


    def __init__(self, exception_class, message):
        self._exception_class = exception_class
        self.message          = message


    def __enter__(self):
        pass


    def __exit__(self, exc_type, exc_value, traceback):
        success = not exc_type
        if success:
            raise KoniraReassertError(
                'Expected an exception of type %s but got none'
                % self._exception_class.__name__)
        else:
            return self.validate_failure(exc_type, exc_value)


    def validate_failure(self, exc_type, exc_value):
        wrong_message_was_raised = (self.message and
                                    self.message != str(exc_value))
        if wrong_message_was_raised:
            raise KoniraReassertError(
                "Expected %s('%s') but got %s('%s')" %
                 (self._exception_class.__name__,
                  str(self.message),
                  exc_type.__name__,
                  str(exc_value)))
        elif issubclass(exc_type, self._exception_class):
            return True
        else:
            pass




import sys
import os
from cStringIO        import StringIO
from konira           import tokenizer
from konira.collector import FileCollector
from konira.runner    import Runner
from konira.exc       import DontReadFromInput
from konira.util      import runner_options
import konira.tools


class KoniraCommands(object):

    konira_help = """
konira: A test runner and DSL testing framework for writing readable, 
descriptive tests.

Run tests:
    konira [/path/to/cases] 
    konira ['/path/to/cases::case description::it description']

Control Options:
    -s, no-capture      Avoids capturing stderr and stdout
    -x, fail            Stops at first fail
    -t, traceback       Shows tracebacks with errors/fails
    -d, dots            Displays '.' for passing and 'F' for failed tests.

Matching Options:
    describe            Matches a case description (needs to be 
                        quoted).
    it                  Matches an 'it' spec (needs to be quoted).

"""

    def __init__(self, argv=None, parse=True, test=False):
        self.test             = test
        self.config           = runner_options
        self._original_stdout = sys.stdout
        self._original_stderr = sys.stderr
        self._original_stdin  = sys.stdin
        self._stderr_buffer   = StringIO()
        self._stdout_buffer   = StringIO()
        
        if argv is None:
            argv = sys.argv
        if parse:
            self.parseArgs(argv)


    def msg(self, msg, stdout=True):
        if stdout:
            sys.stdout.write(msg)
        else:
            sys.stderr.write(msg)
        if not self.test:
            sys.exit(1)


    def test_from_path(self, path):
        spath = path.split('::')
        if len(spath) == 3:
            return dict(
                    path        = spath[0],
                    class_name  = tokenizer.valid_class_name(spath[1]),
                    method_name = tokenizer.valid_method_name(spath[2])
                    )
        elif len(spath) == 2:
            return dict(
                    path       = spath[0],
                    class_name = tokenizer.valid_class_name(spath[1]),
                    )
        else:
            return dict(path = path)


    def path_from_argument(self, argv):
        # Get rid of the executable
        p_argv = argv[1:]
        valid_path = [path for path in p_argv if os.path.exists(os.path.abspath(path.split('::')[0]))]
        if valid_path:
            tests = self.test_from_path(os.path.abspath(valid_path[0]))
            return dict(
                    path        = tests.get('path'),
                    class_name  = tests.get('class_name'),
                    method_name = tests.get('method_name')
                    )
        else:
            return dict(path = os.getcwd())


    def capture(self):
        if self.config['capturing'] is True:
            sys.stdout = self._stdout_buffer
            sys.stdin  = DontReadFromInput()


    def end_capture(self):
        if self.config['capturing'] is True:
            sys.stdout = self._original_stdout
            sys.stdin  = self._original_stdin


    def parseArgs(self, argv):
        # No options for now
        options      = ['no-capture', '-s', 'fail', '-x', '-t', '-d',
                        'dots', 'traceback', 'tracebacks', 'describe', 'it']
        help_options = ['-h', '--h', '--help', 'help']

        # Catch help before anything
        if [i for i in argv if i in help_options]:
            self.msg(self.konira_help)

        # Get a valid path
        path_info   = self.path_from_argument(argv)
        search_path = path_info.get('path')
        self.config['class_name']  = path_info.get('class_name')
        self.config['method_name'] = path_info.get('method_name')

        match = [i for i in argv if i in options]

        if match:
            arg_count = {}
            count_arg = {}
            
            for count, argument in enumerate(argv):
                arg_count[argument] = count
                count_arg[count]    = argument

            # Matches for describe
            if arg_count.get('describe'):
                count = arg_count.get('describe')
                value = count_arg.get(count+1)
                if value:
                    self.config['class_name'] = tokenizer.valid_class_name(value)
                else:
                    self.msg("No valid 'describe' name")
            
            # Matches for it
            if arg_count.get('it'):
                count = arg_count.get('it')
                value = count_arg.get(count+1)
                if value:
                    self.config['method_name'] = tokenizer.valid_method_name(value)
                else:
                    self.msg("No valid 'it' name")

            # Dotted output
            if [opt for opt in ['-d', 'dots'] if opt in match]:
                self.config['dotted'] = True

            # Traceback options
            if [opt for opt in ['-t', 'traceback'] if opt in match]:
                self.config['traceback'] = True

            # Fail options
            if [opt for opt in ['-x', 'fail'] if opt in match]:
                self.config['first_fail'] = True

            # Capturing options
            if [opt for opt in ['-s', 'no-capture'] if opt in match]:
                self.config['capturing'] = False

        test_files = FileCollector(search_path)

        if not test_files:
            self.msg("No cases found to test.")
        
        try:
            self.capture()
            test_runner = Runner(test_files, self.config)
            test_runner.run()
            self.end_capture()

            test_runner.report()
        except KeyboardInterrupt:
            self.msg("Exiting from konira.\n")






import os
import re


class FileCollector(list):


    def __init__(self, path):
        self.path              = path
        self.valid_module_name = re.compile(r'case[_a-z]\w*\.py$', re.IGNORECASE)
        self._collect()


    def _collect(self):
        if os.path.isfile(self.path):
            self.append(self.path)
            return

        # Local is faster
        walk = os.walk
        join = os.path.join
        path = self.path
        levels_deep = 0

        for root, dirs, files in walk(path):
            levels_deep += 1

            # Start checking for Python packages after 3 levels
            if levels_deep > 2:
                if not '__init__.py' in files:
                    continue 
            for item in files:
                absolute_path = join(root, item)
                if not self.valid_module_name.match(item):
                    continue
                self.append(absolute_path)



def globals_from_execed_file(filename):
    globals_ = {}
    try:
        execfile(filename, globals_)
        return globals_
    except SyntaxError, e:
        msg = "konira was unable to load %s." % filename
        raise SyntaxError(msg)
    except Exception, e:
        raise


